package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.stub.pickuprequest.*;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;

import ca.canadapost.cpcdp.pickup.util.ClientUtil;

import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Service;
import javax.xml.ws.soap.SOAPFaultException;

import java.io.File;
import java.net.URL;
import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;

/**
 * Sample code for the UpdatePickupRequest Canada Post service.
 * 
 * The UpdatePickupRequest service modifies the characteristics of a pickup request (i.e. contact, pickup times, etc.).
 * 
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * 
 */
public class UpdatePickupRequestClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_Pickup_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String customer = userProperties.get("mailedBy");
            
            QName serviceQName = new QName("http://www.canadapost.ca/ws/soap/pickuprequest", "PickupRequestService");
            Path wsdlPath = FileSystems.getDefault().getPath("SOAP/wsdl", "pickuprequest.wsdl");
            Service service = Service.create(wsdlPath.toUri().toURL(), serviceQName);
            PickupRequestPortType port = service.getPort(PickupRequestPortType.class);

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/enab/soap/pickuprequest");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));
            
            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            PickupRequestUpdateDetailsType details = new PickupRequestUpdateDetailsType();  
            
            ContactInfoType contactInfo = new ContactInfoType();
            contactInfo.setContactName("Jane Smith");
            contactInfo.setEmail("jane.smith@host.com");
            contactInfo.setContactPhone("123-456-7890");

            LocationDetailsType locationDetails = new LocationDetailsType();
            locationDetails.setFiveTonFlag(false);
            locationDetails.setLoadingDockFlag(false);
            locationDetails.setPickupInstructions("Front door.");
                        
            OnDemandPickupTimeType pickupTime = new OnDemandPickupTimeType();
            DatatypeFactory df = DatatypeFactory.newInstance();
            pickupTime.setDate(df.newXMLGregorianCalendar("2015-05-07"));
            pickupTime.setPreferredTime("12:00");
            pickupTime.setClosingTime("17:00");
            
            PickupTimesType pickupTimes = new PickupTimesType();
            pickupTimes.setOnDemandPickupTime(pickupTime);
            
            details.setContactInfo(contactInfo);
            details.setLocationDetails(locationDetails);
            details.setPickupVolume("25");
            details.setPickupTimes(pickupTimes);
            
            
            UpdatePickupRequestRequest request = new UpdatePickupRequestRequest();
            request.setLocale("FR");
            request.setCustomerNumber(customer);
            request.setRequestId("0074703815");
            request.setPickupRequestUpdate(details);

            //execute request
            UpdatePickupRequestResponse response = port.updatePickupRequest(request);

            //retrieve values from response
            if(response.getMessages() == null){
            	System.out.println("Update Pickup Request Success: " + response.isUpdatePickupRequestSuccess());
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
